Name:       kernel-adaptation-sbj
Summary:    Kernel Adaptation for sbj
Version:    3.4.108.20190226.1
Release:    1
Group:      Kernel/Linux Kernel
License:    GPLv2
URL:        https://bitbucket.org/jolla/qualcomm-linux
Source0:    %{name}-%{version}.tar.bz2
Source5:    README

# Normal kernel build requirements
BuildRequires:  python
BuildRequires:  pkgconfig(ncurses)
BuildRequires:  mer-kernel-checks
BuildRequires:  perl
BuildRequires:  kmod >= 9
BuildRequires:  fdupes
BuildRequires:  droid-tools
BuildRequires:  hw-ramdisk >= 1.0.3
BuildRequires:  initrd-helpers
BuildRequires:  cpio
BuildRequires:  yamui

# Build requirements for perf
BuildRequires:  binutils-devel
BuildRequires:  elfutils-devel
BuildRequires:  python-devel
BuildRequires:  flex
BuildRequires:  bison

# Run time requires for kernel
Requires:	flash-partition
Requires:	kmod

Provides:   kernel = %{version}

# Remember to update this version when ever the oem headers update
# in the droid-system package.
Provides:    droid-oem-headers = 4

Obsoletes:   boston-bootimg

BuildRequires: oneshot
Requires: oneshot
%{_oneshot_requires_post}

%description
Kernel for sbj.

%define kernel_version_build %{version}
%define kernel_devel_dir %{_prefix}/src/linux-%{kernel_version_build}

%define package_dir %{name}-%{version}
%define modules_dir %{buildroot}/lib/modules/%{kernel_version_build}
%define builds_uImage 0
%define builds_vmlinuz 1
%define kernel_arch %{_arch}
%define labeldir /dev/disk/by-partlabel

%package perf
Summary:    Linux profiling with performance counters
Group:      Kernel/Linux kernel
Requires:   binutils

%description perf
%{summary}.

%package devel
Summary:    Devel files for sbj kernel
Group:      Development/System
Requires:   %{name} = %{version}-%{release}
Provides:   kernel-devel = %{version}

%description devel
Devel for sbj kernel

%prep
%setup -q -n %{package_dir}

# Build machinery adds the extraversion so we need to parse it
%define extraver `echo %{version} | sed 's/3.4.108//'`
echo %{extraver}
# make sure EXTRAVERSION says what we want it to say
# For sbj we will change CONFIG_LOCALVERSION in .config file instead
# perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = %{extraver}/" Makefile

# Verify this spec is using the latest template version
/usr/bin/mer_verify_kernel_spec 1 --fatal-if-old

echo The target hw is sbj
echo The desc is %{summary}

%build

make sbj_defconfig

# Set extra version using CONFIG_LOCALVERSION
perl -p -i -e "s/^CONFIG_LOCALVERSION=.*/CONFIG_LOCALVERSION=\"%{extraver}\"/" .config
perl -p -i -e "s/^CONFIG_LOCALVERSION_AUTO.*/# CONFIG_LOCALVERSION_AUTO is not set/" .config

# Verify the config meets the current Mer requirements
# /usr/bin/mer_verify_kernel_config .config

# arm/mips: uImage
# others  : bzImage
%if 0%{?builds_uImage}
make %{?jobs:-j%jobs} uImage
%endif

%if 0%{?builds_vmlinuz}
make %{?jobs:-j%jobs} bzImage
%endif

make %{?jobs:-j%jobs} modules

cd tools/perf
make %{?jobs:-j%jobs}

%install
rm -rf %{buildroot}

# Modules
mkdir -p %{modules_dir}/
make INSTALL_MOD_PATH=%{buildroot} modules_install
touch %{modules_dir}/modules.dep
rm -f %{modules_dir}/source
rm -f %{modules_dir}/build
ln -s %{kernel_devel_dir} %{modules_dir}/build
ln -s build %{modules_dir}/source

# /boot
mkdir -p %{buildroot}/boot/
make INSTALL_PATH=%{buildroot}/boot/ install

%if 0%{?builds_uImage}
install -m 755 arch/%{kernel_arch}/boot/uImage %{buildroot}/boot/
%endif

%if 0%{?builds_vmlinuz}
# Create a boot.img image from the zImage
initrd/create-boot-img.sh %{modules_dir}
install -m 755 initrd/boot.img %{buildroot}/boot/boot.img
%endif

install -m 644 .config %{buildroot}/boot/config-%{kernel_version_build}
install -m 600 System.map %{buildroot}/boot/System.map-%{kernel_version_build}

# And save the headers/makefiles etc for building modules against
#
# This all looks scary, but the end result is supposed to be:
# * all arch relevant include/ files
# * all Makefile/Kconfig files
# * all script/ files

mkdir -p %{buildroot}/%{kernel_devel_dir}

# dirs for additional modules per module-init-tools, kbuild/modules.txt
# first copy everything
cp --parents $(find  -type f -name "Makefile*" -o -name "Kconfig*") %{buildroot}/%{kernel_devel_dir}
cp Module.symvers %{buildroot}/%{kernel_devel_dir}
cp System.map %{buildroot}/%{kernel_devel_dir}
if [ -s Module.markers ]; then
cp Module.markers %{buildroot}/%{kernel_devel_dir}
fi
# then drop all but the needed Makefiles/Kconfig files
rm -rf %{buildroot}/%{kernel_devel_dir}/Documentation
rm -rf %{buildroot}/%{kernel_devel_dir}/scripts
rm -rf %{buildroot}/%{kernel_devel_dir}/include

# Copy all scripts
cp .config %{buildroot}/%{kernel_devel_dir}
cp -a scripts %{buildroot}/%{kernel_devel_dir}
if [ -d arch/%{kernel_arch}/scripts ]; then
cp -a arch/%{kernel_arch}/scripts %{buildroot}/%{kernel_devel_dir}/arch/%{kernel_arch}
fi
# FIXME - what's this trying to do ... if *lds expands to multiple files the -f test will fail.
if [ -f arch/%{kernel_arch}/*lds ]; then
cp -a arch/%{kernel_arch}/*lds %{buildroot}/%{kernel_devel_dir}/arch/%{kernel_arch}/
fi
# Clean any .o files from the 'scripts'
find %{buildroot}/%{kernel_devel_dir}/scripts/ -name \*.o -print0 | xargs -0 rm -f

# arch-specific include files
cp -a --parents arch/%{kernel_arch}/include %{buildroot}/%{kernel_devel_dir}

# arm has include files under plat- and mach- areas (x86/mips don't)
%if "%{?kernel_arch}" == "arm"
cp -a --parents arch/%{kernel_arch}/mach-*/include %{buildroot}/%{kernel_devel_dir}
cp -a --parents arch/%{kernel_arch}/plat-*/include %{buildroot}/%{kernel_devel_dir}
%endif

# normal include files
mkdir -p %{buildroot}/%{kernel_devel_dir}/include

# copy only include/* directories
cp -a $(find include -mindepth 1 -maxdepth 1 -type d) %{buildroot}/%{kernel_devel_dir}/include

# Make sure the Makefile and version.h have a matching timestamp so that
# external modules can be built. Also .conf
touch -r %{buildroot}/%{kernel_devel_dir}/Makefile %{buildroot}/%{kernel_devel_dir}/include/linux/version.h
touch -r %{buildroot}/%{kernel_devel_dir}/.config %{buildroot}/%{kernel_devel_dir}/include/linux/autoconf.h

# Copy .config to include/config/auto.conf so "make prepare" is unnecessary.
cp %{buildroot}/%{kernel_devel_dir}/.config %{buildroot}/%{kernel_devel_dir}/include/config/auto.conf

# We put this in boot for debuginfo script to pick it up to debuginfo package
install -m 744 vmlinux %{buildroot}/boot/vmlinux-%{kernel_version_build}

# mark modules executable so that strip-to-file can strip them
find %{buildroot}/lib/modules/%{kernel_version_build} -name "*.ko" -type f -exec chmod u+x {} \;

install -D -m 755 tools/perf/perf %{buildroot}/%{_sbindir}/perf

# HACKS
find %{buildroot}%{kernel_devel_dir}/ -name '.gitignore' -delete
find %{buildroot}%{kernel_devel_dir}/ -name '.*.cmd' -delete
# rpmlint fails if there is executable rights for these files
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/oem_smem_struct.h
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/qcfg.h
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/qdlog.h
chmod -x %{buildroot}/%{kernel_devel_dir}/include/linux/i2c/tsl2772.h
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/boot_mode.h
chmod -x %{buildroot}/%{kernel_devel_dir}/include/linux/input/l3gd20.h
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/kevent.h
chmod -x %{buildroot}/%{kernel_devel_dir}/include/linux/input/focaltech_ft5316_ts.h
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/qvariables.h
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/oem_log_def.h
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/qfa.h
chmod -x %{buildroot}/%{kernel_devel_dir}/include/linux/input/bma250.h
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/qotp.h
chmod -x %{buildroot}/%{kernel_devel_dir}/include/linux/input/synaptics_s3202_touch_DS4_3_2.h
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/ftags.h
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/partition_softlink.h
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/hwid.h
chmod -x %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/pm_log.h
chmod -x %{buildroot}/%{kernel_devel_dir}/include/linux/input/synaptics_s3202_touch.h
rm -f %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/boot_mode.h
rm -f %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/ftags.h
rm -f %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/partition_softlink.h
rm -f %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/qcfg.h
rm -f %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/qdlog.h
rm -f %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/qfa.h
rm -f %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/qotp.h
rm -f %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/qvariables.h
rm -f %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/oem_smem_struct.h
rm -f %{buildroot}/%{kernel_devel_dir}/arch/arm/mach-msm/include/mach/oem_log_def.h

%if 0%{?builds_vmlinuz}
ln -s System.map-%{kernel_version_build} %{buildroot}/boot/System.map
%endif

%fdupes  %{buildroot}%{kernel_devel_dir}/

mkdir -p  %{buildroot}/lib/modules/%{kernel_version_build}/kernel/net/wireless
# Touch the files so we can %ghost them in %files and remove properly when package is updated.
touch  %{buildroot}/lib/modules/%{kernel_version_build}/modules.{dep,dep.bin,alias,alias.bin,softdep,symbols,symbols.bin,builtin.bin,devname}

install -D -m 700 scripts/flash-bootimg-sbj.sh %{buildroot}/var/lib/platform-updates/flash-bootimg-sbj.sh

%files
%defattr(-,root,root,-)
/lib/modules/%{kernel_version_build}/kernel
/lib/modules/%{kernel_version_build}/modules.builtin
/lib/modules/%{kernel_version_build}/modules.order
/boot/System.map-%{kernel_version_build}
/boot/config-%{kernel_version_build}
%dir /lib/modules
%dir /lib/modules/%{kernel_version_build}

# do we need this? should it be versioned only for x86
#/boot/System.map

%if 0%{?builds_vmlinuz}
/boot/boot.img
/boot/System.map
%endif

%if 0%{?builds_uImage}
/boot/uImage
%endif

%if 0%{?builds_firmware}
/lib/firmware/*
%endif

# We ghost vmlinux so that it does not get into the kernel package, but gets
# picked up by debuginfo packaging script.
%ghost /boot/vmlinux-%{kernel_version_build}

# These files will be updated by depmod later.
%ghost /lib/modules/%{kernel_version_build}/modules.dep
%ghost /lib/modules/%{kernel_version_build}/modules.dep.bin
%ghost /lib/modules/%{kernel_version_build}/modules.alias
%ghost /lib/modules/%{kernel_version_build}/modules.alias.bin
%ghost /lib/modules/%{kernel_version_build}/modules.softdep
%ghost /lib/modules/%{kernel_version_build}/modules.symbols
%ghost /lib/modules/%{kernel_version_build}/modules.symbols.bin
%ghost /lib/modules/%{kernel_version_build}/modules.builtin.bin
%ghost /lib/modules/%{kernel_version_build}/modules.devname
%dir /lib/modules/%{kernel_version_build}/kernel/net/wireless
%dir /var/lib/platform-updates
/var/lib/platform-updates/flash-bootimg-sbj.sh

%files perf
%defattr(-,root,root,-)
%{_sbindir}/perf

%files devel
%defattr(-,root,root,-)
/lib/modules/%{kernel_version_build}/build
/lib/modules/%{kernel_version_build}/source
%{kernel_devel_dir}/*
%{kernel_devel_dir}/.config

%post
# When doing install that is done during the image creation thus we don't add
# the oneshot on install but on all other cases.
if [ $1 -ne 1 ] ; then
  add-preinit-oneshot /var/lib/platform-updates/flash-bootimg-sbj.sh
fi

# Make sure the module information is correct
depmod -a %{kernel_version_build}
